//                       msrdriver.h                     2005-07-06 Agner Fog

// Device driver for access to Model-specific registers and control registers
// in Windows 2000 and later (32 bit x86 platform).

//  2005 GNU General Public License www.gnu.org/copyleft/gpl.html


// commands for MSR driver
enum EMSR_COMMAND {
   MSR_IGNORE = 0,                // do nothing
   MSR_STOP   = 1,                // skip rest of list
   MSR_READ   = 2,                // read model specific register
   MSR_WRITE  = 3,                // write model specific register
   CR_READ    = 4,                // read control register
   CR_WRITE   = 5,                // write control register
   UNUSED1    = 0x7fffffff        // make sure this enum takes 32 bits
};


// input/output data structure for MSR driver
struct SMSRInOut {
   EMSR_COMMAND msr_command;      // command for read or write register
   unsigned int register_number;  // register number
   union {
      __int64 value;              // 64 bit value to read or write
      unsigned int val[2];        // lower and upper 32 bits
   };
};


#ifndef _SIZE_T_DEFINED
#ifdef  _WIN64
typedef unsigned __int64 size_t;
#else
typedef unsigned int     size_t;
#endif
#define _SIZE_T_DEFINED
#endif

// functions defined in MSRDriver2.asm
extern "C"  __int64 ReadMSR (int r);  // read model specific register
extern "C"  void WriteMSR (int r, __int64 value); // write model specific register
extern "C"  size_t ReadCR(int r);                 // read control register
extern "C"  void WriteCR(int r, size_t value);    // write control register
