////// PMCTest.h /////////////////////////////////////////  2006-08-14 AgF //
//
//                    Header file for PMC Test program
//
// This header file is included by PMCTestA.cpp and PMCTestB.cpp. Please see
// these files for description of the program.
//
// This header file contains class declarations, function prototypes, 
// constants and other definitions for this program.
//
//////////////////////////////////////////////////////////////////////////////
#pragma once

#include <windows.h>
#include <windef.h>
#include <winnt.h>
#include <winsvc.h>
#include <stdlib.h>
#include <stdio.h>

// maximum number of performance counters used
const int MAXCOUNTERS = 4;

// maximum number of repetitions
const int MAXREPEAT = 128;

// max name length of counters
const int COUNTERNAMELEN = 10;
 

// list of input/output data structures for MSR driver
#define MAX_QUE_ENTRIES 32        // maximum number of entries in queue

class CMSRInOutQue {
public:
   // constructor
   CMSRInOutQue();
   // put record in queue
   int put (EMSR_COMMAND msr_command, unsigned int register_number,
      unsigned int value_lo, unsigned int value_hi = 0);
   // list of entries
   SMSRInOut queue[MAX_QUE_ENTRIES];
   // get size of queue
   int GetSize () {return n;}
protected:
   // number of entries
   int n;
};


// class CMSRDriver encapsulates the interface to the driver MSRDriver32.sys
// which is needed for privileged access to set up the MSRs. This class
// loads, unloads and sends commands to MSRDriver
class CMSRDriver {
public:
	CMSRDriver();             // constructor
	~CMSRDriver();            // destructor
   LPCTSTR GetDriverName();  // get name of driver
   // send commands to driver to read or write MSR registers
	int AccessRegisters(void * pnIn, int nInLen, void * pnOut, int nOutLen);
   // send commands to driver to read or write MSR registers
   int AccessRegisters(CMSRInOutQue & q);
   // read performance monitor counter
   // send command to driver to read one MSR register
   int MSRRead(int r);
   // send command to driver to write one MSR register
   int MSRWrite(int r, int val);
   // send command to driver to read one control register
   size_t CRRead(int r);
   // send command to driver to write one control register
   int CRWrite(int r, size_t val);
	int LoadDriver();         // load MSRDriver
	int UnloadDriver();       // unload MSRDriver
protected:
   int Need64BitDriver();    // tell whether we need 32 bit or 64 bit driver
	SC_HANDLE scm;
	SC_HANDLE service;
	HANDLE hDriver;
   LPCTSTR DriverFileName;
   LPCTSTR DriverSymbolicName;
};


// codes for processor family
enum EProcFamily {
   UNKNOWN = 0,              // unknown. can't do performance monitoring
   INTEL_P1MMX = 1,          // Intel Pentium 1 or Pentium MMX
   INTEL_P23   = 2,          // Pentium Pro, Pentium 2, Pentium 3
   INTEL_PM    = 4,          // Pentium M, Core, Core 2
   INTEL_P4    = 8,          // Pentium 4 and Pentium 4 with EM64T
   INTEL_P23M  = 0x16,       // Pentium Pro, Pentium 2, Pentium 3, Pentium M, Core1
   INTEL_CORE  = 0x10,       // Intel Core Solo/Duo
   INTEL_CORE2 = 0x20,       // Intel Core 2

   AMD_ATHLON = 0x1000,      // AMD Athlon
   AMD_ATHLON64 = 0x2000,    // AMD Athlon 64 or Opteron
   AMD = (0x1000|0x2000),    // AMD Athlon or Athlon 64 or Opteron
   UNUSED2 = 0x7fffffff
};


// record specifying how to count a particular event on a particular CPU family
struct SCounterDefinition {
   int          CounterType;                // what to count
   EProcFamily  ProcessorFamily;            // processor family. values may be OR'ed
   int          CounterFirst, CounterLast;  // counter number or a range of possible alternative counter numbers
   int          EventSelectReg;             // event select register
   int          Event;                      // event code
   int          EventMask;                  // event mask
   char         Description[COUNTERNAMELEN];// name of counter. length must be < COUNTERNAMELEN
};


// class CCounters defines, starts and stops MSR counters
class CCounters {
public:
   CCounters();                             // constructor
   char * DefineCounter(int CounterType);   // request a counter setup
   char * DefineCounter(SCounterDefinition & CounterDef); // request a counter setup
   void StartCounters();                    // start counting
   void StopCounters();                     // stop and reset counters
   CMSRDriver msr;                          // interface to MSR access driver
   char * CounterNames[MAXCOUNTERS];        // name of each counter
protected:
   CMSRInOutQue queue1;                     // que of MSR commands to do by StartCounters()
   CMSRInOutQue queue2;                     // que of MSR commands to do by StopCounters()
   // translate event select number to register address for P4 processor:
   static int GetP4EventSelectRegAddress(int CounterNr, int EventSelectNo); 
   int NumCounterDefinitions;               // number of possible counter definitions in table CounterDefinitions
   EProcFamily family;                      // microprocessor family
};   


// get microprocessor family
extern "C" EProcFamily GetProcessorFamily();

// the basic test loop containing the code to test
extern "C" int TestLoop ();


//////////////////////////////////////////////////////////////////////////////
//    Global variables. May be accessed from assembly module
//////////////////////////////////////////////////////////////////////////////

// number of performance counters used
extern "C" int NumCounters, MaxNumCounters;

// counter registers used
extern "C" int Counters[MAXCOUNTERS];
extern "C" int EventRegistersUsed[MAXCOUNTERS];

// count results
extern "C" int ClockOverhead, PMCOverhead[MAXCOUNTERS]; // counts by the test program itself
extern "C" int ClockTemp, PMCTemp[MAXCOUNTERS]; // counts before test
extern "C" int ClockResults[MAXREPEAT]; // clock counts after test minus counts before test
extern "C" int PMCResults[MAXREPEAT][MAXCOUNTERS]; // PMC counts after test minus counts before test

// list of counter definitions
extern "C" SCounterDefinition CounterDefinitions[];

// list of desired counter types
extern "C" int CounterTypesDesired[MAXCOUNTERS];
